*! version 3.1   -  9/10/08 - pbe - handle continuous variables
*! version 3.0   -  2/19/08 - pbe - complete rewrite - extend to multifactorial unbalanced designs
*! version 2.0   -  5/19/06 - pbe
*! name change   - 10/26/05 - pbe
*! version 1.1   -  2/26/02 - pbe
*! version 1.0   -  3/01/01 - pbe

program define anovacontrast
  version 8.0

  tempvar grpvar

  if ~inlist(e(cmd),"anova","regress") {
    display as error "Can't find anova results"
    exit
  } 
  syntax varlist(max=1) [if], Values(numlist) ///
       [ Title(string) NU(real 0) SS(real 0) ]

  local depvar = e(depvar)
  local varnames = e(varnames)
  
  local contin = e(cmdline)
  local strloc = strpos("`contin'","cont")
  if `strloc'>0 {
    local contin = substr("`contin'",`strloc'+1,.)
    local strloc = strpos("`contin'","(")
    local strloc2 = strpos("`contin'",")")
    local contin = substr("`contin'",`strloc'+1,`strloc2'-`strloc'-1)
    local wdcount = wordcount("`contin'")
    forvalues i = 1/`wdcount' {
      local wdtemp = word("`contin'",`i')
      local varnames = subinword("`varnames'","`wdtemp'","",.)
    }
  }
  
  local rss = `ss'
  if `rss' == 0 { 
     local rss = `e(rss)' 
  }
  local dfr = `nu'
  if `dfr' == 0 { 
     local dfr = `e(df_r)' 
  }
  egen `grpvar' = group(`varlist')
  preserve
  collapse (mean) mdv=`depvar' mgv=`grpvar' (count) ndv=`depvar' `if' `in', by(`varnames')
  mkmat mgv mdv ndv, matrix(cm)
  restore

  local mc = wordcount("`values'")
  quietly tab `varlist'
  if r(r) != `mc' {
		error 121
  }
  
  mat conlist=J(1,`mc',0)
  local ctot=0
  forvalues i = 1/`mc' {
    local cval : word `i' of `values'
    mat conlist[1, `i'] = `cval'
    local ctot = `ctot' + `cval'
  }  

local c2=0
local psi=0
local bign=0
local rows=rowsof(cm)
mat matj=J(`rows',1,0)
forvalues i = 1/`rows' {
  mat matj[`i',1] = conlist[1,cm[`i',1]]
  local psi = `psi' + matj[`i',1]*cm[`i',2]
  local c2 = `c2' + matj[`i',1]*matj[`i',1]/cm[`i',3]
  local bign = `bign' + cm[`i',3]
}
  local cadj = `rows'/`mc'
  local ss = (`psi'/sqrt(`c2'))^2
  local msr = `rss'/`dfr'
  local df = 1
  local fratio = `ss'/`msr'

  display _newline
   if "`title'" ~= "" { 
       display in green "`title'" 
   }
  if `ctot'~=0 {
    display in yellow "Warning: contrast weights do not sum to zero, contrast may not be valid"
  }
  display in green "Contrast variable: " in yellow "`varlist'" in green " (" in yellow "`values'" in green ")"  " Dep Var: " in yellow "`depvar'"
  display in green "source           SS          df      MS"     _col(48)  "Contrast = " in yellow %8.2f = `psi'/`cadj' 
  display in green "---------+---------------------------------" _col(48) "N of obs = " in yellow %8.0f = `bign'
  display in green "contrast |" in yellow  _col(11) %11.0g = `ss'     _col(26) %6.0g = `df' _col(32) %12.4f = `ss'      in green _col(48) "F        = "  in yellow %8.2f = `fratio'
  display in green "error    |" in yellow  _col(11) %11.0g = `rss' _col(26)  %6.0g = `dfr'  _col(32) %12.4f = `msr' in green _col(48) "Prob > F = " in yellow %8.4f = fprob(1,`dfr',`fratio')
  display in green "---------+---------------------------------"  _col(48) "t        = " in yellow %8.2f = sqrt(`fratio')

  matrix drop cm matj conlist 

end

